from abc import ABCMeta, abstractmethod
from pathlib import Path
from typing import NamedTuple

import requests
import requests_cache
from decouple import config
from rich.console import Console
from rich.table import Table
from rich.text import Text

ONE_DAY_IN_SECONDS = 24 * 60 * 60
TIMEOUT = 5

console = Console()
error_console = Console(stderr=True, style="bold red")

HOME_DIR = str(Path.home())
CACHE_DB_LOCATION = config("CACHE_DB_LOCATION", default=HOME_DIR)
CACHE_DB_PATH = Path(CACHE_DB_LOCATION) / ".pybites_search_cache.sqlite"
CACHE_EXPIRATION_SECONDS = config(
    "CACHE_EXPIRATION_SECONDS", default=ONE_DAY_IN_SECONDS
)

requests_cache.install_cache(CACHE_DB_PATH, expire_after=CACHE_EXPIRATION_SECONDS)


class ContentPiece(NamedTuple):
    title: str
    url: str


class PybitesSearch(metaclass=ABCMeta):
    @abstractmethod
    def match_content(self, search: str) -> list[ContentPiece]:
        """Search through Pybites content, implement for a specific source"""

    def get_data(self, endpoint):
        return requests.get(endpoint, timeout=TIMEOUT).json()

    def show_header(self, title: str) -> None:
        header = Text(title, style="bold underline")
        console.print(header)

    def show_matches(self, content: list[ContentPiece], extra_nl: bool = False) -> None:
        """Show search results in a nice table"""
        if content:
            table = Table("Title", "Url")
            for row in content:
                table.add_row(row.title, row.url)
            console.print(table)
        else:
            error_console.print("No results found")

        if extra_nl:
            console.print()



import typer

from .article import ArticleSearch
from .bite import BiteSearch
from .podcast import PodcastSearch
from .tip import TipSearch
from .youtube import YouTubeSearch

app = typer.Typer()


@app.command()
def article(search: str):
    searcher = ArticleSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def bite(search: str):
    searcher = BiteSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def podcast(search: str):
    searcher = PodcastSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def tip(search: str):
    searcher = TipSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def video(search: str):
    searcher = YouTubeSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def all(search: str):
    classes = (ArticleSearch, BiteSearch, PodcastSearch, TipSearch, YouTubeSearch)
    titles = (
        "Articles",
        "Bite Exercises",
        "Podcast Episodes",
        "Python Tips",
        "YouTube videos",
    )
    for title, cls_ in zip(titles, classes):
        searcher = cls_()  # type: ignore
        results = searcher.match_content(search)
        title = f"Pybites {title}"
        searcher.show_header(title)
        searcher.show_matches(results, extra_nl=True)



from .base import ContentPiece, PybitesSearch

PODCAST_ENDPOINT = "https://codechalleng.es/api/podcasts/"
PODCAST_BASE_URL = "https://www.pybitespodcast.com/1501156/"


class PodcastSearch(PybitesSearch):
    def match_content(self, search: str) -> list[ContentPiece]:
        entries = self.get_data(PODCAST_ENDPOINT)
        results = []
        for entry in entries:
            if search.lower() in (entry["title"] + entry["description"]).lower():
                results.append(
                    ContentPiece(
                        title=entry["title"],
                        url=PODCAST_BASE_URL + entry["slug"],
                    )
                )
        return results


if __name__ == "__main__":
    searcher = PodcastSearch()
    results = searcher.match_content("cms")
    searcher.show_matches(results)



from .base import ContentPiece, PybitesSearch

BITES_ENDPOINT = "https://codechalleng.es/api/bites/"
PLATFORM_BASE_URL = "https://codechalleng.es/bites/"


class BiteSearch(PybitesSearch):
    def match_content(self, search: str) -> list[ContentPiece]:
        entries = self.get_data(BITES_ENDPOINT)
        results = []
        for entry in entries:
            if search.lower() in (entry["title"] + entry["description"]).lower():
                results.append(
                    ContentPiece(
                        title=entry["title"],
                        url=f"{PLATFORM_BASE_URL}{entry['number']}",
                    )
                )
        return results


if __name__ == "__main__":
    searcher = BiteSearch()
    results = searcher.match_content("fastapi")
    searcher.show_matches(results)



from .base import ContentPiece, PybitesSearch

ARTICLE_ENDPOINT = "https://codechalleng.es/api/articles/"


class ArticleSearch(PybitesSearch):
    def match_content(self, search: str) -> list[ContentPiece]:
        entries = self.get_data(ARTICLE_ENDPOINT)
        results = []
        for entry in entries:
            if search.lower() in (entry["title"] + entry["summary"]).lower():
                results.append(
                    ContentPiece(
                        title=entry["title"],
                        url=entry["link"],
                    )
                )
        return results


if __name__ == "__main__":
    searcher = ArticleSearch()
    results = searcher.match_content("django")
    searcher.show_matches(results)
